-----------------------------------------------------------------------------------------------
-- Client Lua Script for ZInterrupt
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "CombatFloater"
 
-----------------------------------------------------------------------------------------------
-- ZInterrupt Module Definition
-----------------------------------------------------------------------------------------------
local ZInterrupt = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ZInterrupt:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function ZInterrupt:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ZInterrupt OnLoad
-----------------------------------------------------------------------------------------------
function ZInterrupt:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ZInterrupt.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- ZInterrupt OnDocLoaded
-----------------------------------------------------------------------------------------------
function ZInterrupt:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "InterruptProgress", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self.time = 0
	    self.wndMain:Show(false, true)
		self.editMode = false
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterEventHandler("TargetUnitChanged", "OnChangedTarget", self)
		Apollo.RegisterTimerHandler("PrintStartupMessage", "OnStartup", self)
		Apollo.RegisterTimerHandler("UpdateTheBar", "OnTurtle", self)
		Apollo.RegisterSlashCommand("zi", "OnZInterruptOn", self)
		Apollo.RegisterSlashCommand("zinterval", "OnChangeZInterval", self)


		-- Do additional Addon initialization here
		Apollo.CreateTimer("PrintStartupMessage", 3, false)
		Apollo.StartTimer("PrintStartupMessage")
		self.interval = 0.1
		Apollo.CreateTimer("UpdateTheBar", self.interval, true)
		Apollo.StartTimer("UpdateTheBar")

		if (self.restoreData ~= nil) then
			self.wndMain:SetAnchorOffsets(unpack(self.restoreData.offsets))
			self.wndMain:SetAnchorPoints(unpack(self.restoreData.anchors))
			self:ChangeInterval(self.restoreData.interval)
		end
	end
end

-----------------------------------------------------------------------------------------------
-- ZInterrupt Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/zi"
function ZInterrupt:OnZInterruptOn()
	self.editMode = not self.editMode
	self.wndMain:Show(self.editMode, true)
	self.wndMain:SetStyle("Moveable", self.editMode)
	self.wndMain:SetStyle("Sizable", self.editMode)
	self.wndMain:SetStyle("Escapable", not self.editMode)
	if (self.editMode) then
		self.wndMain:FindChild("InterruptCount"):SetText("edit")
	else
		self.wndMain:FindChild("InterruptCount"):SetText("nil")
	end
end

function ZInterrupt:OnChangedTarget()
	self.target = GameLib.GetTargetUnit()
	if (self.target == nil or self.target:IsFriend()) then
		return
	end
	self.targetMaxArmor = self.target:GetInterruptArmorMax()
end

function ZInterrupt:OnTurtle()
	if (self.editMode) then
		return
	end
	if (self.target == nil or self.target:IsFriend()) then
		self.wndMain:Show(false, true)
		return
	end
	local castPercent = self.target:GetCastTotalPercent()
	local interruptArmor = self.target:GetInterruptArmorValue()
	local maxArmor = self.target:GetInterruptArmorMax()
	if (castPercent == nil or interruptArmor == nil or maxArmor == nil) then
		return
	end
	if (castPercent > 0 and castPercent < 100 and interruptArmor < maxArmor) then
		self.wndMain:FindChild("InterruptCount"):SetText(interruptArmor .. "/" .. maxArmor)
		self.wndMain:Show(true, true)
		self.wndMain:SetMax(100)
		self.wndMain:SetProgress(castPercent)
	else
		self.wndMain:Show(false, true)
	end
end

function ZInterrupt:OnChangeZInterval(strCmd, strArg)
	if (strArg == "help") then
		ChatSystemLib.PostOnChannel(2, "Use '/zinterval <number>' to change the interval between bar updates.")
		ChatSystemLib.PostOnChannel(2, "'/zinterval 0.01' means the interrupt bar will update 100 times per second, drastically improving the smoothness of the bar.")
		ChatSystemLib.PostOnChannel(2, "'/zinterval 1' means it will update once per second (very bad).")
		ChatSystemLib.PostOnChannel(2, "Lower values will take slightly more processing power.")
		return
	end
	local newint = tonumber(strArg)
	if (newint == nil or newint < 0.01 or newint > 1) then
		ChatSystemLib.PostOnChannel(2, "Invalid interval value. Must be a number between 0.01 and 1. Default is 0.25.")
		return
	end
	self:ChangeInterval(newint)
end

function ZInterrupt:OnStartup()
	ChatSystemLib.PostOnChannel(2, "ZInterrupt loaded. Type '/zi' to enter and exit edit mode. Type '/zinterval help' to see how to affect the smoothness of the interrupt bar.")
end

function ZInterrupt:OnSave(eLevel)
	if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character) then
		return nil
	end
	local data = {
		anchors = {self.wndMain:GetAnchorPoints()},
		offsets = {self.wndMain:GetAnchorOffsets()},
		interval = self.interval,
	}
	return data
end

function ZInterrupt:OnRestore(eLevel, tData)
	if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character) then
		return nil
	end
	self.restoreData = tData
end

function ZInterrupt:ChangeInterval(newint)
	self.interval = newint
	Apollo.StopTimer("UpdateTheBar")
	Apollo.CreateTimer("UpdateTheBar", self.interval, true)
	Apollo.StartTimer("UpdateTheBar")
end

-----------------------------------------------------------------------------------------------
-- ZInterrupt Instance
-----------------------------------------------------------------------------------------------
local ZInterruptInst = ZInterrupt:new()
ZInterruptInst:Init()
